@@ -43,8 +43,12 @@ class DoublyLinkedList(TailedLinkedList):
         node.__dict__[self.prev_name] = prev_node
 
     def get_node(self, index: int):
-        r = self.regularize_index(index)
-        return self.get_prev(node=self.tail, step=index) if index < r else self.get_next(node=self.head, step=r)
+        steps_from_head = self.regularize_index(index)
+        steps_from_tail = self.size - 1 - steps_from_head
+        if steps_from_head < steps_from_tail:  # take the shorter path
+            return self.get_next(node=self.head, step=steps_from_head)
+        else:
+            return self.get_prev(node=self.tail, step=steps_from_tail)
 
     def __iter__(self):
         """ from index 0 (tail) to -1 (head) """
@@ -82,7 +86,7 @@ class DoublyLinkedList(TailedLinkedList):
             if prev_node is not None:
                 self.set_next(node=prev_node, next_node=next_node)
             if next_node is not None:
-                self.set_prev(node=next_node, next_node=prev_node)
+                self.set_prev(node=next_node, prev_node=prev_node)
         self.size -= 1
 
     def __str__(self) -> str:
@@ -210,9 +214,9 @@ class DoublyLinkedList(TailedLinkedList):
             self.clear()
         else:
             if not (self.head is None and self.new_node().match(node)) and not (self.head is not None and self.head.match(node)):
-                raise ValueError(f"Invalid node with attributes: {list(node.__dict__.keys())}")
+                raise ValueError(f"Invalid node with attributes: {list(node.__dict__)}")
             node_copy = self.new_node(data=self.get_data(node))
-            self.head, self.size = node_copy
+            self.head, self.size = node_copy, 1
             self.tail = node_copy
             while self.get_next(node) is not None:
                 node = self.get_next(node)
@@ -231,23 +235,61 @@ class DoublyLinkedList(TailedLinkedList):
             self.prev_name = other.prev_name  # diff from parent class
             self.copy_from_node(other.head)
 
-    def reverse(self):
-        node = self.tail
-        while node is not None:
-            prev_node = self.get_prev(node)
-            next_node = self.get_next(node)
-            self.set_prev(node=node, prev_node=next_node)
-            self.set_next(node=node, next_node=prev_node)
-            node = next_node
-        self.head, self.tail = self.tail, self.head
-        return self
-
     def has_cycle(self, node) -> bool:
         raise NotImplementedError
 
     def get_cycle_entrance(self, node):
         raise NotImplementedError
 
+    def reverse(self, start: int = 0, end: int = -1, group_size: int = None, remainder_on_left: bool = False):
+        """ To Do: searching for starting node can be optimized (from head vs from tail) """
+        if len(self) < 2:
+            return self
+        rstart = self.regularize_index(end)
+        rend = self.regularize_index(start)
+        if rstart == rend or self.head is None:
+            return self
+        if rend < rstart:
+            raise ValueError(f"Invalid start {start} or end {end}")
+        range_size = rend - rstart + 1
+        group_size = range_size if group_size is None else min(group_size, range_size)
+        remainder_size = range_size % group_size
+        predecessor, node = None, self.head
+        for _ in range(rstart):
+            predecessor, node = node, self.get_next(node)
+        tmp_head = predecessor  # save the tmp head
+        predecessor, node = node, self.get_next(node)  # move forward one more time
+        tmp_tail = predecessor  # save the tmp tail
+        first_loop = True
+        while group_size > 0 and range_size >= group_size:  # group_size > 0 is required, cause range_size can be 0
+            if first_loop and not remainder_on_left and remainder_size > 0:
+                first_loop, shift_size = False, remainder_size
+            else:
+                shift_size = group_size
+            for _ in range(shift_size - 1):
+                successor = self.get_next(node)
+                self.set_next(node=node, next_node=predecessor)
+                self.set_prev(node=predecessor, prev_node=node)  # diff from super().reverse()
+                predecessor, node = node, successor
+            self.set_next(node=tmp_tail, next_node=node)
+            if node is None:
+                self.tail = tmp_tail
+            else:
+                self.set_prev(node=node, prev_node=tmp_tail)  # diff from super().reverse()
+            if tmp_head is None:
+                self.head = predecessor
+                self.set_prev(node=predecessor, prev_node=None)  # diff from super().reverse()
+            else:
+                self.set_next(node=tmp_head, next_node=predecessor)
+                self.set_prev(node=predecessor, prev_node=tmp_head)  # diff from super().reverse()
+            tmp_head, tmp_tail, predecessor, node = tmp_tail, node, node, self.get_next(node)
+            range_size -= shift_size
+            group_size = min(group_size, range_size)
+        return self
+
+
+
+
 
 
 
