@@ -1,9 +1,38 @@
-from collections.abc import MutableMapping
+from collections.abc import Mapping, MutableMapping
+from enum import Enum
 
+from ezcode.Tree.BinaryTreeIterator import BinaryTreeIterator
 from ezcode.Tree.RedBlackTree import RedBlackTree
 from ezcode.Tree.BinarySearchTree import BinarySearchTree
 
 
+class TreeMapIterator:
+    Mode = Enum("Mode", ["KEY", "VALUE", "ITEM"])
+
+    def __init__(self, tree, reverse=False, mode=Mode.KEY):
+        self.mode = mode
+        self.iterator = BinaryTreeIterator(
+            node=tree.root,
+            mode=BinaryTreeIterator.Mode.IN_ORDER,
+            is_left_first=(not reverse),
+            data_name=tree.data_name,
+            left_name=tree.left_name,
+            right_name=tree.right_name
+        )
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        if self.mode == TreeMapIterator.Mode.KEY:
+            return next(self.iterator).key
+        elif self.mode == TreeMapIterator.Mode.VALUE:
+            return next(self.iterator).value
+        else:
+            entry = next(self.iterator)
+            return entry.key, entry.value
+
+
 class TreeMap(MutableMapping):
     class Entry:
         def __init__(self, key, value=None):
@@ -58,41 +87,26 @@ class TreeMap(MutableMapping):
             node.data.value = value
 
     def __iter__(self):
-        return iter(self.tree)  # node.data -> entry(key, value)
+        return TreeMapIterator(tree=self.tree, reverse=False, mode=TreeMapIterator.Mode.KEY)
 
     def __reversed__(self):
-        return reversed(self.tree)
+        return TreeMapIterator(tree=self.tree, reverse=True, mode=TreeMapIterator.Mode.KEY)
 
-    def __repr__(self):
-        return repr(list(iter(self)))
-
-    def keys(self, reverse=False):
-        if reverse:
-            for entry in reversed(self):
-                yield entry.key
-        else:
-            for entry in self:
-                yield entry.key
-
-    def values(self, reverse=False):
-        if reverse:
-            for entry in reversed(self):
-                yield entry.value
-        else:
-            for entry in self:
-                yield entry.value
-
-    def items(self, reverse=False):
-        if reverse:
-            for entry in reversed(self):
-                yield entry.key, entry.value
-        else:
-            for entry in self:
-                yield entry.key, entry.value
+    def __str__(self):
+        return "{" + ", ".join([f"{key}: {value}" for key, value in self.items()]) + "}"
 
     def clear(self):
         self.tree.clear()
 
+    def keys(self, reverse=False):
+        yield from TreeMapIterator(tree=self.tree, reverse=reverse, mode=TreeMapIterator.Mode.KEY)
+
+    def values(self, reverse=False):
+        yield from TreeMapIterator(tree=self.tree, reverse=reverse, mode=TreeMapIterator.Mode.VALUE)
+
+    def items(self, reverse=False):
+        yield from TreeMapIterator(tree=self.tree, reverse=reverse, mode=TreeMapIterator.Mode.ITEM)
+
     def pop(self, key):
         parents, node = self.tree.search(data=TreeMap.Entry(key, None), track_parents=True)
         if node is None:
@@ -106,3 +120,10 @@ class TreeMap(MutableMapping):
             raise KeyError("Pop from empty map")
         return self.tree.pop(reverse=reverse)
 
+    def update(self, other):
+        if not isinstance(other, Mapping):
+            raise TypeError
+        for key, value in other.items():
+            self[key] = value
+
+
